@@ -1,5 +1,5 @@
 /*
-Copyright (C) 1997-2017 Erwin Waterlander
+Copyright (C) 1997-2019 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,7 +20,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#ifndef KEY_RESIZE
 #include <signal.h>
+#endif
 #include <assert.h>
 #include "wcd.h"
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
@@ -844,9 +846,18 @@ void displayRefresh(int init);
  * to just rebuild the windows from scratch */
 void displayResize()
 {
-   /* end curses mode */
-   endwin();
-   refresh();  /* start curses */
+#ifdef __PDCURSES__
+   /* Correct resizing on Windows (ConEmu and Windows 10 console) requires a
+      recent PDCurses version ( >= 3.7 ). */
+   resize_term(0,0);
+#else  
+   /* resize_term() is not working properly with old versions of curses.
+      (e.g. seen with ncurses 5.6.20080804). There are still a lot of old
+      curses library installations around. Stopping and restarting curses
+      does the job too with old curses libraries. */
+   endwin();  /* end curses mode */
+   refresh(); /* start curses */
+#endif
 
    wcd_display.scrollWinHeight = LINES - INPUT_WIN_HEIGHT;
    wcd_display.lines_per_page = wcd_display.scrollWinHeight;
@@ -876,6 +887,7 @@ void displayResize()
    displayRefresh(1);
 }
 
+#ifndef KEY_RESIZE
 #if CAN_RESIZE
 void signalSigwinchDisplay (int sig)
 {
@@ -883,6 +895,7 @@ void signalSigwinchDisplay (int sig)
   signal(SIGWINCH,signalSigwinchDisplay);
 }
 #endif
+#endif
 
 
 void wcd_mvwaddstr(WINDOW *win, int x, int y, char *str)
@@ -1191,13 +1204,13 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
   int i, n=0, c=0;
   int displayed_list;
   size_t len ;
-#ifndef __PDCURSES__
   SCREEN *sp;
-#endif
 
+#ifndef KEY_RESIZE
 #if CAN_RESIZE
    signal (SIGWINCH, signalSigwinchDisplay);
 #endif
+#endif
 
 /* Notice that list->size > 1 when this function is called. */
 
@@ -1237,16 +1250,12 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 
 /* Older versions of PDCurses and ncurses < 5.9.20120922 do not
  * support newterm() on Windows */
-#if defined(__PDCURSES__) || (defined(_WIN32) && !defined(__CYGWIN__))
-   initscr();
-#else
    sp = newterm(NULL,stdout,stdin);
    if (sp == NULL)
    {
       print_error("%s", _("Error opening terminal, falling back to stdout interface.\n"));
       return WCD_ERR_CURSES;
    }
-#endif
 
    keypad(stdscr, TRUE);
    intrflush(stdscr, FALSE);
@@ -1261,6 +1270,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
    if (LINES < 4)
    {
       endwin();
+      delscreen(sp);
 #ifdef XCURSES
       XCursesExit();
 #endif
@@ -1286,8 +1296,14 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
    else
       if (ws != NULL)
          len = maxLengthStack(ws);
-      else
+      else {
+         endwin();
+         delscreen(sp);
+#ifdef XCURSES
+         XCursesExit();
+#endif
          return(WCD_ERR_LIST);
+      }
 
    refresh();
 
@@ -1295,6 +1311,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
    if (wcd_display.scrollWin == NULL)
    {
       endwin();
+      delscreen(sp);
 #ifdef XCURSES
       XCursesExit();
 #endif
@@ -1308,7 +1325,9 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 
    if (wcd_display.inputWin == NULL)
    {
+      delwin(wcd_display.scrollWin);
       endwin();
+      delscreen(sp);
 #ifdef XCURSES
       XCursesExit();
 #endif
@@ -1404,6 +1423,9 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
          break;
       case KEY_F (5):
       case Key_CTRL ('l'):
+#ifdef KEY_RESIZE
+      case KEY_RESIZE:
+#endif
             displayResize ();
          break;
      case 3:  /* Control-C */
@@ -1454,7 +1476,10 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
      displayRefresh(0);
    }
 
+   delwin(wcd_display.scrollWin);
+   delwin(wcd_display.inputWin);
    endwin();
+   delscreen(sp);
 #ifdef XCURSES
    XCursesExit();
 #endif
